# -*- coding: utf-8 -*-
#  FriendlyAlbum - Photo Album for Plone
#  Copyright (c) 2006 IngeniWeb - Badredine Ladjemi <badredine.ladjemi@ingeniweb.com>
#  $Id: album.py,v 1.9 2006/05/23 12:09:31 bladjemi Exp $
# 
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation; either version 2 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program; if not, write to the Free Software
#  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#  
from AccessControl import ClassSecurityInfo
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.utils import getToolByName
import simplejson
import copy
import zipfile

vocab_durations = DisplayList(((1, "1 second"), (2, "2 seconds"), (3, "3 seconds"), (5, "5 seconds")))
DEFAULT_SLIDESHOW_DURATION = 2

class FriendlyImage(BaseContent):
    archetype_name = portal_type = meta_type = 'FriendlyImage'
    _at_rename_after_creation = True
    security = ClassSecurityInfo()
	
    schema = BaseSchema + Schema((
	ImageField('image',
		   required=True,
		   sizes={'thumb': (300, 300), 'tiny': (60, 60)}
		   )
	))

    #we want description in base_edit
    schema['description'].metadata = False
    schema['description'].schemata = 'default'

    actions = (
        {
        'id'           : 'view',
        'name'         : 'View',
        'action'       : "string:${object_url}/friendlyimage_view",
        'permissions'  : (CMFCorePermissions.View, ),
        'category'      : 'object',
	'visible'       : True,
        },
    )

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_afterAdd')
    def manage_afterAdd(self, item, container):
	super(FriendlyImage, self).manage_afterAdd(item, container)
	container.registerImage(item)
	
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_beforeDelete')	
    def manage_beforeDelete(self, item, container):
	container.unregisterImage(item)
	super(FriendlyImage, self).manage_beforeDelete(item, container)
	
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setTitle')
    def setTitle(self, value):
        """We have to override setTitle here to return Title"""
	self.getField('title').set(self, value)
	self.getParentNode().registerImage(self)
	return self.Title()

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'setDescription')
    def setDescription(self, value):
	"""We have to override setDescription here to return Description"""
	self.getField('description').set(self, value)
	self.getParentNode().registerImage(self)
	return self.Description()

registerType(FriendlyImage)        	

class FriendlyAlbum(OrderedBaseFolder):
    """
    FriendlyAlbum is a photos container with a nice UI
    """
    _at_rename_after_creation = True
    security = ClassSecurityInfo()
    archetype_name = portal_type = meta_type = 'FriendlyAlbum'

    schema = OrderedBaseFolderSchema + Schema((
	BooleanField('hide_infobox',
		     required=True,
		     default=True,
		     widget=BooleanWidget(
					  description_msgid="help_description_hide_infobox",
					  description="Choose to display or not the information box",
					  label_msgid="help_label_hide_infobox",
					  label="Hide the information box",
					  i18n_domain="friendlyalbum",
					 )
	           ),
	
	           IntegerField('slideshow_duration',
				required=False,
				default=DEFAULT_SLIDESHOW_DURATION,
				vocabulary=vocab_durations,
				widget=SelectionWidget(
	                                               label_msgid="help_label_slideshowduration",
						       label="Slideshow's duration interval",
						       description_msgid="help_description_slideshowduration",
						       description="Time in seconds between each photo during a slideshow",
						       i18n_domain="friendlyalbum",
						      ),
				)
	))

    allowed_content_types = ('FriendlyImage', )
    immediate_view = 'base_view'
    global_allow = 1
    filter_content_types = 1
    
    actions = (
        {
        'id'           : 'view',
        'name'         : 'View',
        'action'       : 'string:${folder_url}/friendlyalbum_view',
        'permissions'  : (CMFCorePermissions.View, ),
        'category'      : 'object',
	'visible'       : True,
        },
        {
	'id'           : 'batch',
	'name'         : 'Edit photos',
	'action'       : 'string:${folder_url}/friendlyalbum_batch',
	'permissions'  : (CMFCorePermissions.ModifyPortalContent, ),
	'category'      : 'object',
	'visible'       : True,
	},
	{
	'id'            : 'upload',
	'name'          : 'Upload photos',
	'action'        : 'string:${folder_url}/friendlyalbum_upload',
	'permissions'   : (CMFCorePermissions.ModifyPortalContent, ),
	'category'      : 'object',
	'visible'       : True,
	},
    )
 
    displayContentTabs = False

    max_upload = 10000000 #in bytes
    max_photos = 50

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'manage_afterAdd')
    def manage_afterAdd(self, item, container):
	super(OrderedBaseFolder, self).manage_afterAdd(item, container)
	self._json_dump = {}
	
    security.declareProtected(CMFCorePermissions.View, 'registerImage')
    def registerImage(self, image, field_name='image'):
	"""
	register an image into the pre-json dictionary
	"""
	root_url = self.absolute_url()
	item = {}

	published_methods = {'title' : 'pretty_title_or_id', 'description' : 'Description'}
	sizes = {'thumb' : "image_tiny", 'small' : "image_thumb", 'normal' : ''}

	item['id'] = image.id
	
	for method in published_methods:
	    item[method] = getattr(image, published_methods[method])()
	    
	for size in sizes:
	    item[size] = '/'.join([root_url, image.id, field_name, sizes[size]])
	    
	self._json_dump[image.id] = item
	return item

    security.declareProtected(CMFCorePermissions.View, 'unregisterImage')
    def unregisterImage(self, image):
	"""
	unregister image from the pre-json dictionary
	"""
	del self._json_dump[image.id]

    security.declareProtected(CMFCorePermissions.View, 'getLastJson')
    def getLastJson(self, item):
	"""
	return the last pre-json dictionary
	"""
	if self._json_dump.has_key(item.id):
	    el = self._json_dump[item.id]
	else:
	    el = self.registerImage(item)
	return el
	
    security.declareProtected(CMFCorePermissions.View, 'listPhotos')
    def listPhotos(self):
        """ lists the photos in the album """
        return self.getFolderContents(contentFilter={"portal_type" : ("FriendlyImage",)}, full_objects=True)
    
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'rebuildJSON')
    def rebuildJSON(self):
        """
	rebuild JSON
        """
	for image in self.listPhotos():
	    self.registerImage(image)	
							    
    security.declareProtected(CMFCorePermissions.View, 'json')
    def json(self):
	"""JSON is a format to exchange data between javascript and any other language that supports it.
	Javascript supports it natively because it's in fact a javascript array, less code and less memory consumption!
	Python supports it by using the simplejson package (don't use python-json, it is buggy and broken)

	cf:
	http://www.crockford.com/JSON/
	
	"""
	plone_utils = getToolByName(self, 'plone_utils')
	self.REQUEST.RESPONSE.setHeader('Content-Type', 'text/x-json;charset=%s' % plone_utils.getSiteEncoding())

	json = simplejson.dumps({'duration': self.getField('slideshow_duration').get(self), 'hide_infobox': self.getField('hide_infobox').get(self), 'photos': [self.getLastJson(item) for item in self.listPhotos()]}, ensure_ascii=False)
	self._json_fragment = json
	return self._json_fragment

    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'reorder')
    def reorder(self, order):
	"""
	reorder the folder with the order given by the list of objects
	"""
	for i in range(len(order)):
	    self.moveObjectToPosition(order[i], i)
        
	plone_utils = getToolByName(self, 'plone_utils')
	plone_utils.reindexOnReorder(self)
			

	plone_utils = getToolByName(self, 'plone_utils')
	plone_utils.reindexOnReorder(self)
	
	
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'deletePhoto')
    def deletePhoto(self, id):
	"""
	delete a photo given its id
	"""
	del self._json_dump[i]
	return self.manage_delObjects([id])
	
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'addPhoto')
    def addPhoto(self, filename, data=None, file_obj=None):
	"""
	add a photo
	"""
	if self.isFull():
	    return None
	    
	#generate a normalized id
	plone_utils = getToolByName(self, 'plone_utils')
	filename = plone_utils.normalizeString(filename)
	
	#if the id is already taken we generate a new one
	if hasattr(self, filename):
	    n = 1
	    oldfilename = filename
	    filename = ''.join([oldfilename, '_', str(n)])
	    while hasattr(self, filename):
		filename = ''.join([oldfilename, '_', str(n)])
		n += 1
	    
	self.invokeFactory(id=filename, type_name='FriendlyImage')

	value = data or file_obj
	getattr(self, filename).setImage(value)
	return filename
   
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'uploadZip')
    def uploadZip(self, zipped):
	"""
	decompress and upload the file
	"""
	if len(self.listPhotos()) >= self.max_photos:
	    return []

	zipped = zipfile.ZipFile(zipped, 'r')
	filenames = []
	for filename in zipped.namelist():
	    name = filename.lower()
	    if name.endswith('.jpg') or name.endswith('.jpeg') or name.endswith('.png') or name.endswith('.gif'):
		filenames.append(self.addPhoto(filename=filename, data=zipped.read(filename)))
	return filenames

    security.declareProtected(CMFCorePermissions.View, 'isFull')
    def isFull(self):
	"""
	True if the album is full, False either
	"""
	return len(self.listPhotos())>=self.max_photos
	
registerType(FriendlyAlbum)
